/* ************************************************************************** */
// Programmer: Tobin Schwaiger-Hastanan (tobin@13hex.com)
// 
// Copyright (C) 2003 13Hex, Inc.
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions 
// are met:
// 
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
// 
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution. 
// 
//      * Neither the name of 13Hex, Inc. nor the names 
//        of its contributors may be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission. 
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
// COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
// POSSIBILITY OF SUCH DAMAGE.
/* ************************************************************************** */
#ifndef _ENTITY_DOT_H_INCLUDED_
#define _ENTITY_DOT_H_INCLUDED_

namespace snbfrx
{
    // forward declaratin of the EntityFactory class.
    class EntityFactory;

    /**
      * Abstract representation of a dynamic entity.
      */
    class Entity
    {
    public:
        /**
          * Entity constructor.
          *
          * @param pfactory pointer to an EntityFactory that created this entity.
          */
        Entity( EntityFactory* pfactory );
    
        /**
          * Virtual destructor.
          */
        virtual ~Entity();

        /**
          * Sets the EntityFactory of the entity factory that created this
          * entity.
          *
          * @param pfactory pointer to an EntityFactory that created this entity.
          */
        void setFactory( EntityFactory* pfactory ) { m_pfactory = pfactory; }

        /**
          * Returns the entity factory that had created this entity.
          *
          * @return EntityFactory pointer to factory that had created this entity.
          */
        EntityFactory* getFactory() const { return m_pfactory; }

        /**
          * Pure virtual funtion that handles the entity update.
          *
          * Typically implementations of this method should handle things
          * like state management, AI updates, position/movement updates,
          * and collision detection for the entity.
          *
          * If the entity has expired/died, this method will return false 
          * and the EntityManager managing this entity will handle the
          * destruction of the entity by handing it off to the EntityFactory's
          * resleaseEntity method.
          *
          * @return boolean value that specifies the "life" status of an entity.
          *         true  - entity is still alive and kicking.
          *         false - entity has died and should be released/destroyed.
          */
        virtual bool update() = 0;

        /**
          * Pure virtual function that handles the entity rendering.
          *
          * This method is called by an EntityManager object when an 
          * entity is alive and associated to that manager.          
          */
        virtual void render() = 0;

    private:
        /** pointer to factory that had created this entity */
        EntityFactory* m_pfactory;
    };
}

#endif